@@ -1,4 +1,4 @@
-/* 
+/*
  *	HT Editor
  *	elf_analy.cc
  *
@@ -25,14 +25,14 @@
 #include "analy_ppc.h"
 #include "analy_register.h"
 #include "analy_x86.h"
-#include "global.h"
+#include "analy_arm.h"
 #include "elf_analy.h"
 
 #include "htctrl.h"
 #include "htdebug.h"
 #include "htiobox.h"
 #include "htelf.h"
-#include "htstring.h"
+#include "strtools.h"
 #include "pestruct.h"
 #include "snprintf.h"
 #include "x86asm.h"
@@ -48,8 +48,12 @@ extern "C" {
 /*
  *
  */
- 
-void ElfAnalyser::init(ht_elf_shared_data *Elf_shared, ht_streamfile *File)
+
+ElfAnalyser::ElfAnalyser()
+{
+}
+
+void ElfAnalyser::init(ht_elf_shared_data *Elf_shared, File *File)
 {
 	elf_shared = Elf_shared;
 	file = File;
@@ -105,10 +109,18 @@ void ElfAnalyser::beginAnalysis()
 			addComment(secaddr, 0, "");
 			addComment(secaddr, 0, ";******************************************************************");
 			addComment(secaddr, 0, blub);
-			ht_snprintf(blub, sizeof blub, ";  virtual address  %08x  virtual size   %08x", s32->sh_addr, s32->sh_size);
+			if (c32) {
+				ht_snprintf(blub, sizeof blub, ";  virtual address  %08x  virtual size   %08x", s32->sh_addr, s32->sh_size);
+			} else {
+				ht_snprintf(blub, sizeof blub, ";  virtual address  %08qx  virtual size   %08qx", s64->sh_addr, s64->sh_size);
+			}
 			addComment(secaddr, 0, blub);
 			if (validAddress(secaddr, scinitialized)) {
-				ht_snprintf(blub, sizeof blub, ";  file offset      %08x  file size      %08x", s32->sh_offset, s32->sh_size);
+				if (c32) {
+					ht_snprintf(blub, sizeof blub, ";  file offset      %08x  file size      %08x", s32->sh_offset, s32->sh_size);
+				} else {
+					ht_snprintf(blub, sizeof blub, ";  file offset      %08qx  file size      %08qx", s64->sh_offset, s64->sh_size);
+				}
 			} else {
 				ht_snprintf(blub, sizeof blub, ";  section is not in file");
 			}
@@ -117,11 +129,11 @@ void ElfAnalyser::beginAnalysis()
 
 			// mark end of sections
 			ht_snprintf(blub, sizeof blub, ";  end of section <%s>", getSegmentNameByAddress(secaddr));
-			Address *secend_addr = (Address *)secaddr->duplicate();
+			Address *secend_addr = secaddr->clone();
 			if (c32) {
 				secend_addr->add(s32->sh_size);
 			} else {
-				secend_addr->add(s64->sh_size.lo);
+				secend_addr->add(s64->sh_size);
 			}
 			newLocation(secend_addr)->flags |= AF_FUNCTION_END;
 			addComment(secend_addr, 0, "");
@@ -130,7 +142,13 @@ void ElfAnalyser::beginAnalysis()
 			addComment(secend_addr, 0, ";******************************************************************");
 
 			validarea->add(secaddr, secend_addr);
-
+			
+			Address *seciniaddr = secaddr->clone();
+			seciniaddr->add(c32 ? s32->sh_size : s64->sh_size);
+			if (validAddress(secaddr, scinitialized) && validAddress(seciniaddr, scinitialized)) {
+				initialized->add(secaddr, seciniaddr);
+			}
+			delete seciniaddr;
 			delete secend_addr;
 		}
 		delete secaddr;
@@ -170,7 +188,7 @@ void ElfAnalyser::beginAnalysis()
 			addComment(entry, 0, ";  executable entry point");
 			break;
 		default:
-			addComment(entry, 0, ";  entry point");               
+			addComment(entry, 0, ";  entry point");
 		}
 		addComment(entry, 0, ";****************************");
 		delete entry;
@@ -188,223 +206,113 @@ void ElfAnalyser::beginAnalysis()
 void ElfAnalyser::initInsertFakeSymbols()
 {
 	if (!elf_shared->undefined2fakeaddr) return;
-	sectionAndIdx *key = NULL;
-	ht_data_uint32 *value;
-	while ((key = (sectionAndIdx*)elf_shared->undefined2fakeaddr->enum_next(
-	(ht_data**)&value, key))) {
-		Address *address = createAddress32(value->value);
-		FILEOFS h = elf_shared->sheaders.sheaders32[key->secidx].sh_offset;
-		ELF_SYMBOL32 sym;
-		file->seek(h+key->symidx*sizeof (ELF_SYMBOL32));
-		file->read(&sym, sizeof sym);
-		create_host_struct(&sym, ELF_SYMBOL32_struct, elf_shared->byte_order);
 
-		FILEOFS sto = elf_shared->sheaders.sheaders32[
-			elf_shared->sheaders.sheaders32[key->secidx].sh_link].sh_offset;
-		file->seek(sto+sym.st_name);
-		char *name = fgetstrz(file);
+	foreach(FakeAddr, fa, *elf_shared->undefined2fakeaddr, {
+		Address *address = createAddress32(fa->addr);
+		FileOfs h = elf_shared->sheaders.sheaders32[fa->secidx].sh_offset;
+		ELF_SYMBOL32 sym;
+		file->seek(h + fa->symidx * sizeof (ELF_SYMBOL32));
+		file->readx(&sym, sizeof sym);
+		createHostStruct(&sym, ELF_SYMBOL32_struct, elf_shared->byte_order);
+
+		FileOfs sto = elf_shared->sheaders.sheaders32[
+			elf_shared->sheaders.sheaders32[fa->secidx].sh_link].sh_offset;
+		file->seek(sto + sym.st_name);
+		char *name = file->fgetstrz();
 		char buf[1024];
 		ht_snprintf(buf, sizeof buf, "undef_%s", name);
 		free(name);
 		make_valid_name(buf, buf);
 		assignSymbol(address, buf, label_func);
-	}
+	})
 }
 
 void ElfAnalyser::initInsertSymbols(int shidx)
 {
 	char elf_buffer[1024];
 	if (elf_shared->ident.e_ident[ELF_EI_CLASS] == ELFCLASS32) {
-		FILEOFS h = elf_shared->sheaders.sheaders32[shidx].sh_offset;
-		FILEOFS sto = elf_shared->sheaders.sheaders32[elf_shared->sheaders.sheaders32[shidx].sh_link].sh_offset;
+		FileOfs h = elf_shared->sheaders.sheaders32[shidx].sh_offset;
+		FileOfs sto = elf_shared->sheaders.sheaders32[elf_shared->sheaders.sheaders32[shidx].sh_link].sh_offset;
 		uint symnum = elf_shared->sheaders.sheaders32[shidx].sh_size / sizeof (ELF_SYMBOL32);
 
 		int *entropy = random_permutation(symnum);
-		for (uint i=0; i<symnum; i++) {
+		for (uint i = 0; i < symnum; i++) {
 			ELF_SYMBOL32 sym;
 			if (entropy[i] == 0) continue;
 			file->seek(h+entropy[i]*sizeof (ELF_SYMBOL32));
 			file->read(&sym, sizeof sym);
-			create_host_struct(&sym, ELF_SYMBOL32_struct, elf_shared->byte_order);
+			createHostStruct(&sym, ELF_SYMBOL32_struct, elf_shared->byte_order);
 
 			file->seek(sto+sym.st_name);
-			char *name = fgetstrz(file);
+			char *name = file->fgetstrz();
 			if (!name) continue;
 
 			switch (sym.st_shndx) {
-				case ELF_SHN_UNDEF:
-					break;
-				case ELF_SHN_ABS:
-					break;
-				case ELF_SHN_COMMON:
-					break;
-				default: {
-					// sym.st_shndx
-					break;
-				}
+			case ELF_SHN_UNDEF:
+				break;
+			case ELF_SHN_ABS:
+				break;
+			case ELF_SHN_COMMON:
+				break;
+			default:
+				// sym.st_shndx
+				break;
 			}
 
-			char *bind;
+			const char *bind;
 			switch (ELF32_ST_BIND(sym.st_info)) {
-				case ELF_STB_LOCAL:
-					bind="local";
-					break;
-				case ELF_STB_GLOBAL:
-					bind="global";
-					break;
-				case ELF_STB_WEAK:
-					bind="weak";
-					break;
-				default:
-					bind="?";
-					break;
+			case ELF_STB_LOCAL:
+				bind="local";
+				break;
+			case ELF_STB_GLOBAL:
+				bind="global";
+				break;
+			case ELF_STB_WEAK:
+				bind="weak";
+				break;
+			default:
+				bind="?";
+				break;
 			}
 
 			switch (ELF32_ST_TYPE(sym.st_info)) {
-				case ELF_STT_NOTYPE:
-				case ELF_STT_FUNC: {
-					char *label = name;
-					if (!getSymbolByName(label)) {
-						elf32_addr sym_addr = sym.st_value;
-						if (elf_shared->shrelocs && sym.st_shndx > 0 && sym.st_shndx < elf_shared->sheaders.count
-						&& elf_shared->shrelocs[sym.st_shndx].relocAddr) {
-							sym_addr += elf_shared->shrelocs[sym.st_shndx].relocAddr;
-						}
-						Address *address = createAddress32(sym_addr);
-						if (validAddress(address, scvalid)) {
-							char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
-							if (!demangled) demangled = cplus_demangle_v3(label, DMGL_PARAMS | DMGL_ANSI | DMGL_TYPES);
-							make_valid_name(label, label);
-
-							ht_snprintf(elf_buffer, sizeof elf_buffer, "; function %s (%s)", (demangled) ? demangled : label, bind);
-
-							if (demangled) free(demangled);
-
-							addComment(address, 0, "");
-							addComment(address, 0, ";********************************************************");
-							addComment(address, 0, elf_buffer);
-							addComment(address, 0, ";********************************************************");
-							pushAddress(address, address);
-							assignSymbol(address, label, label_func);
-						}
-						delete address;
+			case ELF_STT_NOTYPE:
+			case ELF_STT_FUNC: {
+				char *label = name;
+				if (!getSymbolByName(label)) {
+					elf32_addr sym_addr = sym.st_value;
+					if (elf_shared->shrelocs && sym.st_shndx > 0 && sym.st_shndx < elf_shared->sheaders.count
+					&& elf_shared->shrelocs[sym.st_shndx].relocAddr) {
+						sym_addr += elf_shared->shrelocs[sym.st_shndx].relocAddr;
 					}
-					break;
-				}
-				case ELF_STT_OBJECT: {
-					char *label = name;
-					if (!getSymbolByName(label)) {
-						Address *address = createAddress32(sym.st_value);
-						if (validAddress(address, scvalid)) {
-
-							char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
-					
-							make_valid_name(label, label);
-					
-							ht_snprintf(elf_buffer, sizeof elf_buffer, "; data object %s, size %d (%s)", (demangled) ? demangled : label, sym.st_size, bind);
-
-							if (demangled) free(demangled);
-
-							addComment(address, 0, "");
-							addComment(address, 0, ";********************************************************");
-							addComment(address, 0, elf_buffer);
-							addComment(address, 0, ";********************************************************");
-							assignSymbol(address, label, label_data);
-						}
-						delete address;
-					}
-					break;
-				}
-				case ELF_STT_SECTION:
-				case ELF_STT_FILE:
-					break;
-			}
-			free(name);
-		}
-		if (entropy) free(entropy);
-	} else {
-		// FIXME: 64 bit
-		FILEOFS h = elf_shared->sheaders.sheaders64[shidx].sh_offset.lo;
-		FILEOFS sto = elf_shared->sheaders.sheaders64[elf_shared->sheaders.sheaders64[shidx].sh_link].sh_offset.lo;
-		uint symnum = elf_shared->sheaders.sheaders64[shidx].sh_size.lo / sizeof (ELF_SYMBOL64);
-
-		int *entropy = random_permutation(symnum);
-		for (uint i=0; i<symnum; i++) {
-			ELF_SYMBOL64 sym;
-			if (entropy[i] == 0) continue;
-			file->seek(h+entropy[i]*sizeof (ELF_SYMBOL64));
-			file->read(&sym, sizeof sym);
-			create_host_struct(&sym, ELF_SYMBOL64_struct, elf_shared->byte_order);
-
-			file->seek(sto+sym.st_name);
-			char *name = fgetstrz(file);
-			if (!name) continue;
-
-			switch (sym.st_shndx) {
-				case ELF_SHN_UNDEF:
-					break;
-				case ELF_SHN_ABS:
-					break;
-				case ELF_SHN_COMMON:
-					break;
-				default: {
-					// sym.st_shndx
-					break;
-				}
-			}
-
-			char *bind;
-			switch (ELF64_ST_BIND(sym.st_info)) {
-				case ELF_STB_LOCAL:
-					bind="local";
-					break;
-				case ELF_STB_GLOBAL:
-					bind="global";
-					break;
-				case ELF_STB_WEAK:
-					bind="weak";
-					break;
-				default:
-					bind="?";
-					break;
-			}
-
-			switch (ELF64_ST_TYPE(sym.st_info)) {
-				case ELF_STT_NOTYPE:
-				case ELF_STT_FUNC: {
-					char *label = name;
-					if (!getSymbolByName(label)) {
-						Address *address = createAddress64(sym.st_value);
-
+					Address *address = createAddress32(sym_addr);
+					if (validAddress(address, scvalid)) {
 						char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
-
+						if (!demangled) demangled = cplus_demangle_v3(label, DMGL_PARAMS | DMGL_ANSI | DMGL_TYPES);
 						make_valid_name(label, label);
-
 						ht_snprintf(elf_buffer, sizeof elf_buffer, "; function %s (%s)", (demangled) ? demangled : label, bind);
-
 						if (demangled) free(demangled);
-
 						addComment(address, 0, "");
 						addComment(address, 0, ";********************************************************");
 						addComment(address, 0, elf_buffer);
 						addComment(address, 0, ";********************************************************");
 						pushAddress(address, address);
 						assignSymbol(address, label, label_func);
-						
-						delete address;
 					}
-					break;
+					delete address;
 				}
-				case ELF_STT_OBJECT: {
-					char *label = name;
-					if (!getSymbolByName(label)) {
-						Address *address = createAddress64(sym.st_value);
-
+				break;
+			}
+			case ELF_STT_OBJECT: {
+				char *label = name;
+				if (!getSymbolByName(label)) {
+					Address *address = createAddress32(sym.st_value);
+					if (validAddress(address, scvalid)) {
 						char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
-					
+
 						make_valid_name(label, label);
-					
-						ht_snprintf(elf_buffer, sizeof elf_buffer, "; data object %s, size %d (%s)", (demangled) ? demangled : label, sym.st_size.lo, bind);
+
+						ht_snprintf(elf_buffer, sizeof elf_buffer, "; data object %s, size %d (%s)", (demangled) ? demangled : label, sym.st_size, bind);
 
 						if (demangled) free(demangled);
 
@@ -413,14 +321,113 @@ void ElfAnalyser::initInsertSymbols(int shidx)
 						addComment(address, 0, elf_buffer);
 						addComment(address, 0, ";********************************************************");
 						assignSymbol(address, label, label_data);
-						
-						delete address;
 					}
-					break;
+					delete address;
 				}
-				case ELF_STT_SECTION:
-				case ELF_STT_FILE:
-					break;
+				break;
+			}
+			case ELF_STT_SECTION:
+			case ELF_STT_FILE:
+				break;
+			}
+			free(name);
+		}
+		if (entropy) free(entropy);
+	} else {
+		// FIXME: 64 bit
+		FileOfs h = elf_shared->sheaders.sheaders64[shidx].sh_offset;
+		FileOfs sto = elf_shared->sheaders.sheaders64[elf_shared->sheaders.sheaders64[shidx].sh_link].sh_offset;
+		uint symnum = elf_shared->sheaders.sheaders64[shidx].sh_size / sizeof (ELF_SYMBOL64);
+
+		int *entropy = random_permutation(symnum);
+		for (uint i=0; i<symnum; i++) {
+			ELF_SYMBOL64 sym;
+			if (entropy[i] == 0) continue;
+			file->seek(h+entropy[i]*sizeof (ELF_SYMBOL64));
+			file->read(&sym, sizeof sym);
+			createHostStruct(&sym, ELF_SYMBOL64_struct, elf_shared->byte_order);
+
+			file->seek(sto+sym.st_name);
+			char *name = file->fgetstrz();
+			if (!name) continue;
+
+			switch (sym.st_shndx) {
+			case ELF_SHN_UNDEF:
+				break;
+			case ELF_SHN_ABS:
+				break;
+			case ELF_SHN_COMMON:
+				break;
+			default: {
+				// sym.st_shndx
+				break;
+			}
+			}
+
+			const char *bind;
+			switch (ELF64_ST_BIND(sym.st_info)) {
+			case ELF_STB_LOCAL:
+				bind="local";
+				break;
+			case ELF_STB_GLOBAL:
+				bind="global";
+				break;
+			case ELF_STB_WEAK:
+				bind="weak";
+				break;
+			default:
+				bind="?";
+				break;
+			}
+
+			switch (ELF64_ST_TYPE(sym.st_info)) {
+			case ELF_STT_NOTYPE:
+			case ELF_STT_FUNC: {
+				char *label = name;
+				if (!getSymbolByName(label)) {
+					Address *address = createAddress64(sym.st_value);
+					if (validAddress(address, scvalid)) {
+						char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
+						if (!demangled) demangled = cplus_demangle_v3(label, DMGL_PARAMS | DMGL_ANSI | DMGL_TYPES);
+						make_valid_name(label, label);
+						ht_snprintf(elf_buffer, sizeof elf_buffer, "; function %s (%s)", (demangled) ? demangled : label, bind);
+						if (demangled) free(demangled);
+						addComment(address, 0, "");
+						addComment(address, 0, ";********************************************************");
+						addComment(address, 0, elf_buffer);
+						addComment(address, 0, ";********************************************************");
+						pushAddress(address, address);
+						assignSymbol(address, label, label_func);
+					}
+					delete address;
+				}
+				break;
+			}
+			case ELF_STT_OBJECT: {
+				char *label = name;
+				if (!getSymbolByName(label)) {
+					Address *address = createAddress64(sym.st_value);
+					char *demangled = cplus_demangle(label, DMGL_PARAMS | DMGL_ANSI);
+
+					make_valid_name(label, label);
+
+					ht_snprintf(elf_buffer, sizeof elf_buffer, "; data object %s, size %qd (%s)", (demangled) ? demangled : label, sym.st_size, bind);
+
+					if (demangled) free(demangled);
+
+					addComment(address, 0, "");
+					addComment(address, 0, ";********************************************************");
+					addComment(address, 0, elf_buffer);
+					addComment(address, 0, ";********************************************************");
+					assignSymbol(address, label, label_data);
+
+					delete address;
+				}
+				break;
+			}
+			case ELF_STT_SECTION:
+			case ELF_STT_FILE:
+				break;
 			}
 			free(name);
 		}
@@ -431,10 +438,10 @@ void ElfAnalyser::initInsertSymbols(int shidx)
 /*
  *
  */
-int ElfAnalyser::load(ht_object_stream *f)
+void ElfAnalyser::load(ObjectStream &f)
 {
     	GET_OBJECT(f, validarea);
-	return Analyser::load(f);
+	Analyser::load(f);
 }
 
 /*
@@ -447,7 +454,7 @@ void ElfAnalyser::done()
 	Analyser::done();
 }
 
-OBJECT_ID ElfAnalyser::object_id() const
+ObjectID ElfAnalyser::getObjectID() const
 {
 	return ATOM_ELF_ANALYSER;
 }
@@ -457,7 +464,7 @@ OBJECT_ID ElfAnalyser::object_id() const
  */
 uint ElfAnalyser::bufPtr(Address *Addr, byte *buf, int size)
 {
-	FILEOFS ofs = addressToFileofs(Addr);
+	FileOfs ofs = addressToFileofs(Addr);
 /*     if (ofs == INVALID_FILE_OFS) {
 		int as = 1;
 	}*/
@@ -468,15 +475,14 @@ uint ElfAnalyser::bufPtr(Address *Addr, byte *buf, int size)
 
 bool ElfAnalyser::convertAddressToELFAddress(Address *addr, ELFAddress *r)
 {
-	if (addr->object_id()==ATOM_ADDRESS_FLAT_32) {
+	if (addr->getObjectID()==ATOM_ADDRESS_FLAT_32) {
 		r->a32 = ((AddressFlat32*)addr)->addr;
 		return true;
-	} else if (addr->object_id()==ATOM_ADDRESS_X86_FLAT_32) {
+	} else if (addr->getObjectID()==ATOM_ADDRESS_X86_FLAT_32) {
 		r->a32 = ((AddressX86Flat32*)addr)->addr;
 		return true;
-	} else if (addr->object_id()==ATOM_ADDRESS_FLAT_64) {
-		r->a64.lo = ((AddressFlat64*)addr)->addr.lo;
-		r->a64.hi = ((AddressFlat64*)addr)->addr.hi;
+	} else if (addr->getObjectID()==ATOM_ADDRESS_FLAT_64) {
+		r->a64 = ((AddressFlat64*)addr)->addr;
 		return true;
 	} else {
 		return false;
@@ -504,7 +510,7 @@ Address *ElfAnalyser::createAddress()
 	return new AddressFlat32();
 }
 
-Address *ElfAnalyser::createAddress32(dword addr)
+Address *ElfAnalyser::createAddress32(uint32 addr)
 {
 	switch (elf_shared->header32.e_machine) {
 		case ELF_EM_386:
@@ -513,7 +519,7 @@ Address *ElfAnalyser::createAddress32(dword addr)
 	return new AddressFlat32(addr);
 }
 
-Address *ElfAnalyser::createAddress64(qword addr)
+Address *ElfAnalyser::createAddress64(uint64 addr)
 {
 	return new AddressFlat64(addr);
 }
@@ -523,11 +529,21 @@ Address *ElfAnalyser::createAddress64(qword addr)
  */
 Assembler *ElfAnalyser::createAssembler()
 {
-	switch (elf_shared->header32.e_machine) {
+	switch (elf_shared->ident.e_ident[ELF_EI_CLASS]) {
+	case ELFCLASS32:
+		switch (elf_shared->header32.e_machine) {
 		case ELF_EM_386:
 			Assembler *a = new x86asm(X86_OPSIZE32, X86_ADDRSIZE32);
 			a->init();
 			return a;
+		}
+	case ELFCLASS64:
+		switch (elf_shared->header64.e_machine) {
+		case ELF_EM_X86_64:
+			Assembler *a = new x86_64asm();
+			a->init();
+			return a;
+		}
 	}
 	return NULL;
 }
@@ -535,10 +551,10 @@ Assembler *ElfAnalyser::createAssembler()
 /*
  *
  */
-FILEOFS ElfAnalyser::addressToFileofs(Address *Addr)
+FileOfs ElfAnalyser::addressToFileofs(Address *Addr)
 {
 	if (validAddress(Addr, scinitialized)) {
-		dword ofs;
+		FileOfs ofs;
 		ELFAddress ea;
 		if (!convertAddressToELFAddress(Addr, &ea)) return INVALID_FILE_OFS;
 		if (!elf_addr_to_ofs(&elf_shared->sheaders, elf_shared->ident.e_ident[ELF_EI_CLASS], ea, &ofs)) return INVALID_FILE_OFS;
@@ -563,8 +579,7 @@ const char *ElfAnalyser::getSegmentNameByAddress(Address *Addr)
 	if (i == elf_shared->fake_undefined_shidx) {
 		strcpy(elf_sectionname, "$$HT_FAKE$$");
 	} else {
-		strncpy(elf_sectionname, elf_shared->shnames[i], 32);
-		elf_sectionname[32] = 0;
+		ht_strlcpy(elf_sectionname, elf_shared->shnames[i], sizeof elf_sectionname);
 	}
 	return elf_sectionname;
 }
@@ -572,9 +587,9 @@ const char *ElfAnalyser::getSegmentNameByAddress(Address *Addr)
 /*
  *
  */
-const char *ElfAnalyser::getName()
+String &ElfAnalyser::getName(String &res)
 {
-	return file->get_desc();
+	return file->getDesc(res);
 }
 
 /*
@@ -606,56 +621,60 @@ void ElfAnalyser::initUnasm()
 		case ELFCLASS64: machine = elf_shared->header64.e_machine; elf64 = true; break;
 	}
 	switch (machine) {
-		case ELF_EM_386: // Intel
-			DPRINTF("initing analy_x86_disassembler\n");
+	case ELF_EM_386:
+		DPRINTF("initing analy_x86_disassembler\n");
+		analy_disasm = new AnalyX86Disassembler();
+		((AnalyX86Disassembler*)analy_disasm)->init(this, elf64 ? ANALYX86DISASSEMBLER_FLAGS_FLAT64 : 0);
+		break;
+	case ELF_EM_X86_64:
+		if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS64) {
+			errorbox("x86_64 cant be used in a 32-Bit ELF.");
+		} else {
 			analy_disasm = new AnalyX86Disassembler();
-			((AnalyX86Disassembler*)analy_disasm)->init(this, elf64 ? ANALYX86DISASSEMBLER_FLAGS_FLAT64 : 0);
-			break;
-		case ELF_EM_IA_64: // Intel ia64
-			if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS64) {
-				errorbox("Intel IA64 cant be used in a 32-Bit ELF.");
-			} else {
-				analy_disasm = new AnalyIA64Disassembler();
-				((AnalyIA64Disassembler*)analy_disasm)->init(this);
-			}
-			break;
-		case ELF_EM_PPC: // PowerPC
-			if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS32) {
-				errorbox("Intel PowerPC cant be used in a 64-Bit ELF.");
-			} else {
-				DPRINTF("initing analy_ppc_disassembler\n");
-				analy_disasm = new AnalyPPCDisassembler();
-				((AnalyPPCDisassembler*)analy_disasm)->init(this);
-			}
-			break;
-		case ELF_EM_PPC64: // PowerPC64
-			if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS64) {
-				errorbox("Intel PowerPC64 cant be used in a 32-Bit ELF.");
-			} else {
-				DPRINTF("initing analy_ppc_disassembler\n");
-				analy_disasm = new AnalyPPCDisassembler();
-				((AnalyPPCDisassembler*)analy_disasm)->init(this);
-			}
-			break;
-		default:
-			DPRINTF("no apropriate disassembler for machine %04x\n", machine);
-			warnbox("No disassembler for unknown machine type %04x!", machine);
+			((AnalyX86Disassembler*)analy_disasm)->init(this, ANALYX86DISASSEMBLER_FLAGS_AMD64 | ANALYX86DISASSEMBLER_FLAGS_FLAT64);
+		}
+		break;
+	case ELF_EM_IA_64: // Intel ia64
+		if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS64) {
+			errorbox("Intel IA64 cant be used in a 32-Bit ELF.");
+		} else {
+			analy_disasm = new AnalyIA64Disassembler();
+			((AnalyIA64Disassembler*)analy_disasm)->init(this);
+		}
+		break;
+	case ELF_EM_PPC: // PowerPC
+		if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS32) {
+			errorbox("PowerPC32 cant be used in a 64-Bit ELF.");
+		} else {
+			DPRINTF("initing analy_ppc_disassembler\n");
+			analy_disasm = new AnalyPPCDisassembler();
+			((AnalyPPCDisassembler*)analy_disasm)->init(this, ANALY_PPC_32);
+		}
+		break;
+	case ELF_EM_PPC64: // PowerPC64
+		if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS64) {
+			errorbox("PowerPC64 cant be used in a 32-Bit ELF.");
+		} else {
+			DPRINTF("initing analy_ppc_disassembler\n");
+			analy_disasm = new AnalyPPCDisassembler();
+			((AnalyPPCDisassembler*)analy_disasm)->init(this, ANALY_PPC_64);
+		}
+		break;
+        case ELF_EM_ARM: // Arm
+                if (elf_shared->ident.e_ident[ELF_EI_CLASS] != ELFCLASS32) {
+                        errorbox("ARM cant be used in a 64-Bit ELF.");
+                } else {
+                        DPRINTF("initing analy_arm_disassembler\n");
+                        analy_disasm = new AnalyArmDisassembler();
+                        ((AnalyArmDisassembler*)analy_disasm)->init(this);
+                }
+                break;
+	default:
+		DPRINTF("no apropriate disassembler for machine %04x\n", machine);
+		warnbox("No disassembler for unknown machine type %04x!", machine);
 	}
 }
 
-/*
- *
- */
-void ElfAnalyser::log(const char *msg)
-{
-	/*
-	 *	log() does to much traffic so dont log
-	 *   perhaps we reactivate this later
-	 *
-	 */
-/*	LOG(msg);*/
-}
-
 /*
  *
  */
@@ -667,7 +686,7 @@ Address *ElfAnalyser::nextValid(Address *Addr)
 /*
  *
  */
-void ElfAnalyser::store(ht_object_stream *f)
+void ElfAnalyser::store(ObjectStream &f) const
 {
 	PUT_OBJECT(f, validarea);
 	Analyser::store(f);
@@ -691,11 +710,11 @@ int ElfAnalyser::queryConfig(int mode)
 /*
  *
  */
-Address *ElfAnalyser::fileofsToAddress(FILEOFS fileofs)
+Address *ElfAnalyser::fileofsToAddress(FileOfs fileofs)
 {
 	ELFAddress ea;
 	if (elf_ofs_to_addr(&elf_shared->sheaders, elf_shared->ident.e_ident[ELF_EI_CLASS], fileofs, &ea)) {
-		switch (elf_shared->ident.e_ident[ELF_EI_CLASS]) {          
+		switch (elf_shared->ident.e_ident[ELF_EI_CLASS]) {
 			case ELFCLASS32: return createAddress32(ea.a32);
 			case ELFCLASS64: return createAddress64(ea.a64);
 		}
@@ -717,40 +736,40 @@ bool ElfAnalyser::validAddress(Address *Addr, tsectype action)
 	if (!convertAddressToELFAddress(Addr, &ea)) return false;
 	if (!elf_addr_to_section(sections, cls, ea, &sec)) return false;
 	switch (cls) {
-		case ELFCLASS32: {
-			ELF_SECTION_HEADER32 *s=sections->sheaders32+sec;
-			switch (action) {
-				case scvalid:
-					return true;
-				case scread:
-					return true;
-				case scwrite:
-				case screadwrite:
-					return s->sh_flags & ELF_SHF_WRITE;
-				case sccode:
-					return (s->sh_flags & ELF_SHF_EXECINSTR) && (s->sh_type==ELF_SHT_PROGBITS);
-				case scinitialized:
-					return s->sh_type==ELF_SHT_PROGBITS;
-			}
-			return false;
+	case ELFCLASS32: {
+		ELF_SECTION_HEADER32 *s = sections->sheaders32 + sec;
+		switch (action) {
+		case scvalid:
+			return true;
+		case scread:
+			return true;
+		case scwrite:
+		case screadwrite:
+			return s->sh_flags & ELF_SHF_WRITE;
+		case sccode:
+			return (s->sh_flags & ELF_SHF_EXECINSTR) && (s->sh_type == ELF_SHT_PROGBITS);
+		case scinitialized:
+			return s->sh_type==ELF_SHT_PROGBITS;
 		}
-		case ELFCLASS64: {
-			ELF_SECTION_HEADER64 *s=sections->sheaders64+sec;
-			switch (action) {
-				case scvalid:
-					return true;
-				case scread:
-					return true;
-				case scwrite:
-				case screadwrite:
-					return s->sh_flags.lo & ELF_SHF_WRITE;
-				case sccode:
-					return (s->sh_flags.lo & ELF_SHF_EXECINSTR) && (s->sh_type==ELF_SHT_PROGBITS);
-				case scinitialized:
-					return s->sh_type==ELF_SHT_PROGBITS;
-			}
-			return false;
+		return false;
+	}
+	case ELFCLASS64: {
+		ELF_SECTION_HEADER64 *s = sections->sheaders64 + sec;
+		switch (action) {
+		case scvalid:
+			return true;
+		case scread:
+			return true;
+		case scwrite:
+		case screadwrite:
+			return s->sh_flags & ELF_SHF_WRITE;
+		case sccode:
+			return (s->sh_flags & ELF_SHF_EXECINSTR) && (s->sh_type == ELF_SHT_PROGBITS);
+		case scinitialized:
+			return s->sh_type==ELF_SHT_PROGBITS;
 		}
+		return false;
+	}
 	}
 	return false;
 }
