@@ -21,27 +21,25 @@
 #ifndef __ASM_H__
 #define __ASM_H__
 
-#include "common.h"
-
-#define CPU_X86	1
+#include "data.h"
 
 #define MAX_INSN_SIZE	16
 
 struct CPU_ADDR {
 	union {
 		struct {
-			word seg;
-			dword offset;
+			uint16 seg;
+			uint32 offset;
 		} addr32;
 		struct {
-			qword addr;
+			uint64 addr;
 		} flat64;
 	};
 };
 
 struct asm_code {
 	asm_code *next;
-	dword size;
+	int size;
 	byte data[MAX_INSN_SIZE];
 	void *context;
 };
@@ -55,9 +53,9 @@ typedef void asm_insn;
 
 class Assembler: public Object {
 protected:
-	int (*imm_eval_proc)(void *context, char **s, dword *v);
+	int (*imm_eval_proc)(void *context, const char *s, uint64 &v);
 	void *imm_eval_context;
-	
+
 	asm_code *codes;
 	asm_code code;
 	char error_msg[256];
@@ -65,26 +63,29 @@ protected:
 	int options;
 	bool bigendian;
 
-			void emitbyte(byte b);
-			void emitword(word w);
-			void emitdword(dword d);
-			void free_asm_codes();
-			void deletecode(asm_code *c);
-			void clearcode();
-			void newcode();
-			void pushcode();
+		void emitbyte(byte b);
+		void emitword(uint16 w);
+		void emitdword(uint32 d);
+		void emitqword(uint64 q);
+		void free_asm_codes();
+		void deletecode(asm_code *c);
+		void clearcode();
+		void newcode();
+		void pushcode();
 public:
 			Assembler(bool bigendian);
-			~Assembler();
+			Assembler(BuildCtorArg&a): Object(a) {};
+	virtual		~Assembler();
+
 /* new */
 	virtual	asm_insn *alloc_insn();
 	virtual	asm_code *encode(asm_insn *asm_insn, int options, CPU_ADDR cur_address);
-			char *get_error_msg();
-	virtual	char *get_name();
-	virtual	int translate_str(asm_insn *asm_insn, const char *s);
-			void set_error_msg(char *format, ...);
-			void set_imm_eval_proc(int (*imm_eval_proc)(void *context, char **s, dword *v), void *imm_eval_context);
-			asm_code *shortest(asm_code *codes);
+		const char *get_error_msg();
+	virtual	const char *get_name();
+	virtual	bool translate_str(asm_insn *asm_insn, const char *s) = 0;
+		void set_error_msg(const char *format, ...);
+		void set_imm_eval_proc(int (*imm_eval_proc)(void *context, const char *s, uint64 &v), void *imm_eval_context);
+		asm_code *shortest(asm_code *codes);
 };
 
 /*
@@ -116,24 +117,25 @@ class Disassembler: public Object {
 protected:
 	int options;
 	bool highlight;
-	
-			const char *get_cs(AsmSyntaxHighlightEnum style);
-			void hexd(char **s, int size, int options, int imm);
-			void enable_highlighting();
-			void disable_highlighting();
+
+		const char *get_cs(AsmSyntaxHighlightEnum style);
+		void hexd(char **s, int size, int options, uint32 imm);
+		void hexq(char **s, int size, int options, uint64 imm);
+		void enable_highlighting();
+		void disable_highlighting();
 public:
-			Disassembler();
-			~Disassembler();
+		Disassembler();
+		Disassembler(BuildCtorArg&a): Object(a) {};
 /* new */
 	virtual	dis_insn *createInvalidInsn();
 	virtual	dis_insn *decode(byte *code, int maxlen, CPU_ADDR cur_address)=0;
 	virtual	dis_insn *duplicateInsn(dis_insn *disasm_insn)=0;
 	virtual	void	getOpcodeMetrics(int &min_length, int &max_length, int &min_look_ahead, int &avg_look_ahead, int &addr_align)=0;
 	virtual	byte getSize(dis_insn *disasm_insn)=0;
-	virtual	char *getName()=0;
+	virtual	const char *getName()=0;
 	virtual	bool selectNext(dis_insn *disasm_insn);
-	virtual	char *str(dis_insn *disasm_insn, int style);
-	virtual	char *strf(dis_insn *disasm_insn, int style, char *format)=0;
+	virtual	const char *str(dis_insn *disasm_insn, int style);
+	virtual	const char *strf(dis_insn *disasm_insn, int style, const char *format)=0;
 	virtual	bool validInsn(dis_insn *disasm_insn)=0;
 };
 
@@ -167,13 +169,15 @@ public:
 #define DISASM_STRF_DEFAULT_FORMAT	"?p#%p #%n\t%1?2#, %2?3/, %3/#"
 #define DISASM_STRF_SMALL_FORMAT	"?p#%p #%n?1- %1?2#,%2?3/,%3/#-"
 
-#define ATOM_DISASM_X86 MAGICD("DIS\x01")
-#define ATOM_DISASM_ALPHA MAGICD("DIS\x02")
-#define ATOM_DISASM_JAVA MAGICD("DIS\x03")
-#define ATOM_DISASM_IA64 MAGICD("DIS\x04")
-#define ATOM_DISASM_IL MAGICD("DIS\x05")
-#define ATOM_DISASM_X86_VXD MAGICD("DIS\x06")
-#define ATOM_DISASM_PPC MAGICD("DIS\x07")
+#define ATOM_DISASM_X86		MAGIC32("DIS\x01")
+#define ATOM_DISASM_ALPHA	MAGIC32("DIS\x02")
+#define ATOM_DISASM_JAVA	MAGIC32("DIS\x03")
+#define ATOM_DISASM_IA64	MAGIC32("DIS\x04")
+#define ATOM_DISASM_IL		MAGIC32("DIS\x05")
+#define ATOM_DISASM_X86_VXD	MAGIC32("DIS\x06")
+#define ATOM_DISASM_PPC		MAGIC32("DIS\x07")
+#define ATOM_DISASM_X86_64	MAGIC32("DIS\x08")
+#define ATOM_DISASM_ARM         MAGIC32("DIS\x09")
 
 #define ASM_SYNTAX_DEFAULT "\\@d"
 #define ASM_SYNTAX_COMMENT "\\@#"
