@@ -20,13 +20,20 @@
  *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "htendian.h"
+#include "endianess.h"
 #include "ppcdis.h"
 #include "ppcopc.h"
+#include "snprintf.h"
 #include "tools.h"
 
-PPCDisassembler::PPCDisassembler()
+PPCDisassembler::PPCDisassembler(int aMode)
 {
+	mode = aMode;
+}
+
+void PPCDisassembler::load(ObjectStream &f)
+{
+	GET_INT32X(f, mode);
 }
 
 dis_insn *PPCDisassembler::decode(byte *code, int maxlen, CPU_ADDR addr)
@@ -36,7 +43,7 @@ dis_insn *PPCDisassembler::decode(byte *code, int maxlen, CPU_ADDR addr)
 	uint32 op;
 	int dialect = -1;
 
-	insn.data = create_host_int(code, 4, big_endian);
+	insn.data = createHostInt(code, 4, big_endian);
 	
 	if (maxlen<4) {
 		insn.valid = false;
@@ -58,14 +65,10 @@ dis_insn *PPCDisassembler::decode(byte *code, int maxlen, CPU_ADDR addr)
 		const byte *opindex;
 		const struct powerpc_operand *operand;
 		bool invalid;
-		bool need_comma;
-		bool need_paren;
 
 		table_op = PPC_OP (opcode->opcode);
-		if (op < table_op) break;
-		if (op > table_op) continue;
 
-		if ((insn.data & opcode->mask) != opcode->opcode || (opcode->flags & dialect) == 0) {
+		if ((insn.data & opcode->mask) != opcode->opcode/* || (opcode->flags & dialect) == 0*/) {
 			continue;
 		}
 
@@ -80,16 +83,12 @@ dis_insn *PPCDisassembler::decode(byte *code, int maxlen, CPU_ADDR addr)
 		if (invalid) continue;
 
 		/* The instruction is valid.  */
-//		fprintf(out, "%s", opcode->name);
 		insn.name = opcode->name;
-//		if (opcode->operands[0] != 0) fprintf(out, "\t");
 
 		/* Now extract and print the operands.  */
-		need_comma = false;
-		need_paren = false;
 		int opidx = 0;
 		for (opindex = opcode->operands; *opindex != 0; opindex++) {
-			uint32 value;
+			sint32 value;
 
 			operand = powerpc_operands + *opindex;
 
@@ -117,54 +116,33 @@ dis_insn *PPCDisassembler::decode(byte *code, int maxlen, CPU_ADDR addr)
 				continue;
 			}
 
-			if (need_comma) {
-				//fprintf(out, ", ");
-				need_comma = false;
-			}
-
-			/* Print the operand as directed by the flags.  */
-			if ((operand->flags & PPC_OPERAND_GPR) != 0) {
+			if (operand->flags & PPC_OPERAND_GPR_0) {
+				if (value) {
+					insn.op[opidx].flags |= PPC_OPERAND_GPR;
+					insn.op[opidx++].reg = value;
+				} else {
+					insn.op[opidx].flags = 0;
+					insn.op[opidx].imm = value;
+				}
+			} else if (operand->flags & PPC_OPERAND_GPR) {
 				insn.op[opidx++].reg = value;
-			} else if ((operand->flags & PPC_OPERAND_FPR) != 0) {
+			} else if (operand->flags & PPC_OPERAND_FPR) {
 				insn.op[opidx++].freg = value;
-			} else if ((operand->flags & PPC_OPERAND_VR) != 0) {
+			} else if (operand->flags & PPC_OPERAND_VR) {
 				insn.op[opidx++].vreg = value;
-			} else if ((operand->flags & PPC_OPERAND_RELATIVE) != 0) {
-				insn.op[opidx++].rel.mem = addr.addr32.offset + value;
+			} else if (operand->flags & PPC_OPERAND_RELATIVE) {
+				if (mode == PPC_MODE_32) {
+					insn.op[opidx++].rel.mem = addr.addr32.offset + value;
+				} else {
+					insn.op[opidx++].rel.mem = addr.flat64.addr + value;
+				}
 			} else if ((operand->flags & PPC_OPERAND_ABSOLUTE) != 0) {
 				insn.op[opidx++].abs.mem = value;
 			} else if ((operand->flags & PPC_OPERAND_CR) == 0 || (dialect & PPC_OPCODE_PPC) == 0) {
-				insn.op[opidx++].imm = value;
+				insn.op[opidx++].imm = (sint64)value;
 			} else {
-				insn.op[opidx++].creg = value;
-				if (operand->bits == 3) {
-					//fprintf(out, "cr%d", value);
-				} else {
+				insn.op[opidx++].creg = value;			}
 
-//					static const char *cbnames[4] = { "lt", "gt", "eq", "so" };
-					int cr;
-					int cc;
-					cr = value >> 2;
-//					if (cr != 0) fprintf(out, "4*cr%d", cr);
-					cc = value & 3;
-					if (cc != 0) {
-//						if (cr != 0) fprintf(out, "+");
-//						fprintf(out, "%s", cbnames[cc]);
-					}
-				}
-			}
-
-			if (need_paren) {
-//				fprintf(out, ")");
-				need_paren = false;
-			}
-
-			if ((operand->flags & PPC_OPERAND_PARENS) == 0) {
-				need_comma = true;
-			} else {
-//				fprintf(out, "(");
-				need_paren = true;
-			}
 		}
 		insn.ops = opidx;
 
@@ -179,7 +157,7 @@ dis_insn *PPCDisassembler::decode(byte *code, int maxlen, CPU_ADDR addr)
 
 dis_insn *PPCDisassembler::duplicateInsn(dis_insn *disasm_insn)
 {
-	ppcdis_insn *insn = (ppcdis_insn *)malloc(sizeof (ppcdis_insn));
+	ppcdis_insn *insn = ht_malloc(sizeof (ppcdis_insn));
 	*insn = *(ppcdis_insn *)disasm_insn;
 	return insn;
 }
@@ -194,17 +172,17 @@ byte PPCDisassembler::getSize(dis_insn *disasm_insn)
 	return ((ppcdis_insn*)disasm_insn)->size;
 }
 
-char *PPCDisassembler::getName()
+const char *PPCDisassembler::getName()
 {
 	return "PPC/Disassembler";
 }
 
-char *PPCDisassembler::str(dis_insn *disasm_insn, int style)
+const char *PPCDisassembler::str(dis_insn *disasm_insn, int style)
 {
 	return strf(disasm_insn, style, "");
 }
 
-char *PPCDisassembler::strf(dis_insn *disasm_insn, int style, char *format)
+const char *PPCDisassembler::strf(dis_insn *disasm_insn, int style, const char *format)
 {
 	if (style & DIS_STYLE_HIGHLIGHT) enable_highlighting();
 	
@@ -215,25 +193,25 @@ char *PPCDisassembler::strf(dis_insn *disasm_insn, int style, char *format)
 	ppcdis_insn *ppc_insn = (ppcdis_insn *) disasm_insn;
 	if (!ppc_insn->valid) {
 		switch (ppc_insn->size) {
-			case 1:
-				strcpy(insnstr, "db         ?");
-				break;
-			case 2:
-				strcpy(insnstr, "dw         ?");
-				break;
-			case 3:
-				strcpy(insnstr, "db         ? * 3");
-				break;
-			case 4:
-				sprintf(insnstr, "dd        %s0x%08x", cs_number, ppc_insn->data);
-				break;
-			default: { /* braces for empty assert */
-				strcpy(insnstr, "?");
-//				assert(0);
-			}
+		case 1:
+			strcpy(insnstr, "db            ?");
+			break;
+		case 2:
+			strcpy(insnstr, "dw            ?");
+			break;
+		case 3:
+			strcpy(insnstr, "db            ? * 3");
+			break;
+		case 4:
+			sprintf(insnstr, "dd           %s0x%08x", cs_number, ppc_insn->data);
+			break;
+		default: { /* braces for empty assert */
+			strcpy(insnstr, "?");
+//			assert(0);
+		}
 		}
 	} else {
-		char *is = insnstr+sprintf(insnstr, "%-10s", ppc_insn->name);
+		char *is = insnstr+sprintf(insnstr, "%-13s", ppc_insn->name);
 		int dialect=-1;
 
 		bool need_comma = false;
@@ -247,7 +225,7 @@ char *PPCDisassembler::strf(dis_insn *disasm_insn, int style, char *format)
 				is += sprintf(is, "%s, ", cs_symbol);
 				need_comma = false;
 			}
-			if ((flags & PPC_OPERAND_GPR) != 0) {
+			if (flags & PPC_OPERAND_GPR) {
 				is += sprintf(is, "%sr%d", cs_default, ppc_insn->op[opidx].reg);
 			} else if ((flags & PPC_OPERAND_FPR) != 0) {
 				is += sprintf(is, "%sf%d", cs_default, ppc_insn->op[opidx].freg);
@@ -255,21 +233,25 @@ char *PPCDisassembler::strf(dis_insn *disasm_insn, int style, char *format)
 				is += sprintf(is, "%svr%d", cs_default, ppc_insn->op[opidx].vreg);
 			} else if ((flags & PPC_OPERAND_RELATIVE) != 0) {
 				CPU_ADDR caddr;
-				caddr.addr32.offset = (dword)ppc_insn->op[opidx].mem.disp;
+				if (mode == PPC_MODE_32) {
+					caddr.addr32.offset = (uint32)ppc_insn->op[opidx].mem.disp;
+				} else {
+					caddr.flat64.addr = ppc_insn->op[opidx].mem.disp;
+				}
 				int slen;
 				char *s = (addr_sym_func) ? addr_sym_func(caddr, &slen, addr_sym_func_context) : 0;
 				if (s) {
 					is += sprintf(is, "%s", cs_default);
-					memmove(is, s, slen);
+					memcpy(is, s, slen);
 					is[slen] = 0;
 					is += slen;
 				} else {
-					is += sprintf(is, "%s0x%x", cs_number, ppc_insn->op[opidx].rel.mem);
+					is += ht_snprintf(is, 100, "%s0x%qx", cs_number, ppc_insn->op[opidx].rel.mem);
 				}
 			} else if ((flags & PPC_OPERAND_ABSOLUTE) != 0) {
-				is += sprintf(is, "%s0x%x", cs_number, ppc_insn->op[opidx].abs.mem);
+				is += ht_snprintf(is, 100, "%s0x%qx", cs_number, ppc_insn->op[opidx].abs.mem);
 			} else if ((flags & PPC_OPERAND_CR) == 0 || (dialect & PPC_OPCODE_PPC) == 0) {
-				is += sprintf(is, "%s%d", cs_number, ppc_insn->op[opidx].imm);
+				is += ht_snprintf(is, 100, "%s%qd", cs_number, ppc_insn->op[opidx].imm);
 			} else if (ppc_insn->op[opidx].op->bits == 3) {
 				is += sprintf(is, "%scr%d", cs_default, ppc_insn->op[opidx].creg);
 			} else {
@@ -302,14 +284,17 @@ char *PPCDisassembler::strf(dis_insn *disasm_insn, int style, char *format)
 	return insnstr;     
 }
 
-OBJECT_ID PPCDisassembler::object_id() const
+ObjectID PPCDisassembler::getObjectID() const
 {
 	return ATOM_DISASM_PPC;
 }
 
+void PPCDisassembler::store(ObjectStream &f) const
+{
+	PUT_INT32X(f, mode);
+}
+
 bool PPCDisassembler::validInsn(dis_insn *disasm_insn)
 {
 	return ((ppcdis_insn*)disasm_insn)->valid;
 }
-
-
