@@ -2,7 +2,7 @@
  *	HT Editor
  *	htcurses.cc (DJGPP implementation)
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -56,7 +56,7 @@ void put_vc(unsigned short *dest, char ch, int vc)
 
 screendrawbuf::screendrawbuf(char *title)
 {
-	bounds b;
+	Bounds b;
 	buf=0;
 	cursorx=0;
 	cursory=0;
@@ -91,7 +91,7 @@ screendrawbuf::~screendrawbuf()
 	}
 }
 
-void screendrawbuf::drawbuffer(drawbuf *b, int x, int y, bounds *clipping)
+void screendrawbuf::drawbuffer(drawbuf *b, int x, int y, Bounds *clipping)
 {
 	drawbufch *ch=b->buf;
 	for (int iy=0; iy<b->size.h; iy++) {
@@ -163,11 +163,11 @@ void screendrawbuf::b_rmove(int rx, int ry)
     /* screens are not movable */
 }
 
-void screendrawbuf::b_setbounds(bounds *b)
+void screendrawbuf::b_setbounds(Bounds *b)
 {
 	genericdrawbuf::b_setbounds(b);
-	if (buf) delete buf;
-	buf=(unsigned short *)malloc(sizeof *buf * size.w * size.h);
+	delete buf;
+	buf = ht_malloc(sizeof *buf * size.w * size.h);
 	b_fill(size.x, size.y, size.w, size.h, VCP(VC_WHITE, VC_BLACK), ' ');
 }
 
