@@ -2,7 +2,7 @@
  *	HT Editor
  *	htcurses.h (DJGPP implementation)
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -73,9 +73,9 @@ public:
 	virtual int b_lprintw(int x, int y, int c, int l, int *text);
 	virtual void b_resize(int rw, int rh);
 	virtual void b_rmove(int rx, int ry);
-	virtual void b_setbounds(bounds *b);
+	virtual void b_setbounds(Bounds *b);
 /* new */
-	void drawbuffer(drawbuf *buf, int x, int y, bounds *clipping);
+	void drawbuffer(drawbuf *buf, int x, int y, Bounds *clipping);
 	void	show();
 	void getcursor(int *x, int *y);
 	void hidecursor();
