@@ -2,7 +2,7 @@
  *	HT Editor
  *	htkeyb.cc (DJGPP implementation)
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -63,7 +63,7 @@ int ht_raw_getkey()
 
 ht_key ht_getkey()
 {
-	UINT r=ht_raw_getkey();
+	uint r=ht_raw_getkey();
 	ht_key k=ht_rawkey2key(r);
 	if ((k==K_INVALID) && (r<=255)) return (ht_key)r;
 	return k;
@@ -75,7 +75,7 @@ struct ht_key_keycode {
 };
 
 ht_key_keycode ht_dj_key_defs[] = {
-	{K_BackSpace,			0x008},
+	{K_Backspace,			0x008},
 	{K_BackTab,			0x10f},
 	
 	{K_Tab,				0x009},
@@ -246,7 +246,7 @@ ht_key_keycode ht_dj_key_defs[] = {
  
 bool init_keyb()
 {
-	UINT i;
+	uint i;
 	
 	for (i=0; i<K_COUNT; i++) {
 		ht_set_key((ht_key)i, -1);
