@@ -2,7 +2,7 @@
  *	HT Editor
  *	htsys.cc (DJGPP implementation)
  *
- *	Copyright (C) 1999-2002 Stefan Weyergraf (stefan@weyergraf.de)
+ *	Copyright (C) 1999-2002 Stefan Weyergraf
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License version 2 as
@@ -128,7 +128,7 @@ int sys_findnext(pfind_t *pfind)
 int sys_pstat(pstat_t *s, const char *filename)
 {
 	char fn[HT_NAME_MAX];
-	strncpy(fn, filename, sizeof fn);
+	ht_strlcpy(fn, filename, sizeof fn);
 	int flen = strlen(fn);
 	if (flen && sys_is_path_delim(fn[flen-1]) && (flen !=3) || (fn[1]!=':')) fn[flen-1] = 0;
 	struct stat st;
@@ -156,7 +156,7 @@ int sys_get_free_mem()
 	return info.available_memory;
 }
 
-int sys_truncate(const char *filename, FILEOFS ofs)
+int sys_truncate(const char *filename, FileOfs ofs)
 {
 	return truncate(filename, ofs);
 }
@@ -186,7 +186,7 @@ int sys_filename_cmp(const char *a, const char *b)
 	return tolower(*a) - tolower(*b);
 }
 
-int sys_ipc_exec(ht_streamfile **in, ht_streamfile **out, ht_streamfile **err, int *handle, const char *cmd, int options)
+int sys_ipc_exec(File **in, File **out, File **err, int *handle, const char *cmd, int options)
 {
 	if (options & HT_IPC_NONBLOCKING) return ENOSYS;
 	int save_stdin = dup(STDIN_FILENO);
@@ -262,7 +262,7 @@ bool sys_write_data_to_native_clipboard(const void *data, int size)
 	if (size > 0xffff) return false;
 	if (!open_clipboard()) return false;
 	int sel;
-	word seg = __dpmi_allocate_dos_memory((size+15)>>4, &sel);
+	uint16 seg = __dpmi_allocate_dos_memory((size+15)>>4, &sel);
 	if (seg == 0xffff) {
 		close_clipboard();
 		return false;
@@ -293,7 +293,7 @@ int sys_get_native_clipboard_data_size()
 	r.x.dx = 0x07; // text
 	__dpmi_int(0x2f, &r);
 	close_clipboard();
-	return ((dword)r.x.dx)<<16+r.x.ax;
+	return ((uint32)r.x.dx)<<16+r.x.ax;
 }
 
 bool sys_read_data_from_native_clipboard(void *data, int max_size)
@@ -305,7 +305,7 @@ bool sys_read_data_from_native_clipboard(void *data, int max_size)
 		return false;
 	}
 	int sel;
-	word seg = __dpmi_allocate_dos_memory((dz+15)>>4, &sel);
+	uint16 seg = __dpmi_allocate_dos_memory((dz+15)>>4, &sel);
 	if (seg == 0xffff) {
 		close_clipboard();
 		return false;
