@@ -23,7 +23,7 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include "htexcept.h"
+#include "except.h"
 extern "C" {
 #include "regex.h"
 }
@@ -42,17 +42,22 @@ bool regmatch(char *str, regex_t *preg, int *len)
 	return false;
 }
 
-bool match_sorted_stringtable(char *s, UINT slen, char **strings, UINT strings_count)
+bool match_sorted_stringtable(char *s, uint slen, const char **strings, uint strings_count)
 {
 	int a=0, b=strings_count-1;
 	int m;
-	while (a<=b) {
-		m=(a+b)>>1;
-		UINT x=strlen(strings[m]);
-		if (slen>x) x=slen;
-		int d=strncmp(s, strings[m], x);
-		if (d<0) b=m-1; else
-			if (d>0) a=m+1; else return true;
+	while (a <= b) {
+		m = (a+b) >> 1;
+		uint x = strlen(strings[m]);
+		if (slen > x) x = slen;
+		int d = ht_strncmp(s, strings[m], x);
+		if (d < 0) {
+			b = m-1; 
+		} else if (d > 0) {
+			a = m+1; 
+		} else {
+			return true;
+		}
 	}
 	return false;
 }
@@ -104,7 +109,7 @@ void ht_lang_syntax_lexer::free_lexer_rules()
 }
 
 
-lexer_token ht_lang_syntax_lexer::gettoken(void *b, UINT buflen, text_pos p, bool start_of_line, lexer_state *ret_state, UINT *ret_len)
+lexer_token ht_lang_syntax_lexer::gettoken(void *b, uint buflen, text_pos p, bool start_of_line, lexer_state *ret_state, uint *ret_len)
 {
 	syntax_lexer_rule *lr=lexer_rules;
 	char *buf = (char*)b;
@@ -126,7 +131,7 @@ lexer_token ht_lang_syntax_lexer::gettoken(void *b, UINT buflen, text_pos p, boo
 					break;
 				}
 				case LRST_STRING: {
-					UINT sl = strlen(lr->string);
+					uint sl = strlen(lr->string);
 					if ((buflen >= sl) && (memcmp(buf, lr->string, sl)==0)) {
 						l = sl;
 					}
@@ -215,19 +220,19 @@ void ht_lang_syntax_lexer::set_lexer_rules(syntax_lexer_rule *lr)
 	while (lexer_rules[lexer_rules_count].needstate) {
 		lexer_rules_count++;
 	}
-	lexer_rules_precompiled=(void**)malloc(sizeof (void**) * lexer_rules_count);
-	for (int i=0; i<lexer_rules_count; i++) {
+	lexer_rules_precompiled = ht_malloc(sizeof (void**) * lexer_rules_count);
+	for (int i=0; i < lexer_rules_count; i++) {
 		if (lexer_rules[i].string_type==LRST_REGEX) {
-			regex_t *preg=(regex_t*)malloc(sizeof (regex_t));
+			regex_t *preg = ht_malloc(sizeof (regex_t));
 
 			/* add an anchor in front of regex */
-			int rl=strlen(lexer_rules[i].string)+1;
-			char *regex=(char*)malloc(1+rl);
+			int rl = strlen(lexer_rules[i].string)+1;
+			char *regex = ht_malloc(1+rl);
 			*regex='^';
-			memmove(regex+1, lexer_rules[i].string, rl);
+			memcpy(regex+1, lexer_rules[i].string, rl);
 			
 			if (regcomp(preg, regex, REG_EXTENDED))
-				throw ht_exception();
+				throw Exception();
 				
 			free(regex);
 			
@@ -313,7 +318,7 @@ syntax_lexer_rule c_syntax_lexer_rules[]={
 	{ 0, 0, LRST_EMPTY, false, false, 0 }
 };
 
-char *c_reserved[]=
+const char *c_reserved[]=
 {
 /* types */
 	"bool", "char", "void", "int", "short", "long",
@@ -344,11 +349,11 @@ char *c_reserved[]=
 void ht_c_syntax_lexer::init()
 {
 	ht_lang_syntax_lexer::init(c_syntax_lexer_rules);
-	c_reserved_sorted=create_sorted_stringtable(c_reserved);
+	c_reserved_sorted = create_sorted_stringtable(c_reserved);
 
-	char **table=c_reserved;
+	const char **table=c_reserved;
 	
-	char **x=table;
+	const char **x=table;
 	while (*x) x++;
 	c_reserved_count=x-table;
 
@@ -360,8 +365,8 @@ void ht_c_syntax_lexer::init()
 
 void ht_c_syntax_lexer::done()
 {
-	if (c_pal.data) free(c_pal.data);
-	if (c_reserved_sorted) free(c_reserved_sorted);
+	free(c_pal.data);
+	free(c_reserved_sorted);
 	ht_lang_syntax_lexer::done();
 }
 
@@ -370,7 +375,7 @@ void ht_c_syntax_lexer::config_changed()
 	reloadpalette();
 }
 
-vcp ht_c_syntax_lexer::getcolor_syntax(UINT pal_index)
+vcp ht_c_syntax_lexer::getcolor_syntax(uint pal_index)
 {
 	return getcolorv(&c_pal, pal_index);
 }
@@ -385,12 +390,12 @@ lexer_state ht_c_syntax_lexer::getinitstate()
 	return LEX_CST_NORMAL;
 }
 
-char *ht_c_syntax_lexer::getname()
+const char *ht_c_syntax_lexer::getname()
 {
 	return "C/C++";
 }
 
-lexer_token ht_c_syntax_lexer::gettoken(void *buf, UINT buflen, text_pos p, bool start_of_line, lexer_state *ret_state, UINT *ret_len)
+lexer_token ht_c_syntax_lexer::gettoken(void *buf, uint buflen, text_pos p, bool start_of_line, lexer_state *ret_state, uint *ret_len)
 {
 	lexer_token t=ht_lang_syntax_lexer::gettoken(buf, buflen, p, start_of_line, ret_state, ret_len);
 	if (t==LEX_CTOK_IDENTIFIER) {
@@ -570,8 +575,8 @@ void ht_html_syntax_lexer::init()
 
 void ht_html_syntax_lexer::done()
 {
-//	if (c_pal.data) free(c_pal.data);
-//	if (c_reserved_sorted) free(c_reserved_sorted);
+//	free(c_pal.data);
+//	free(c_reserved_sorted);
 	ht_lang_syntax_lexer::done();
 }
 
@@ -580,7 +585,7 @@ void ht_html_syntax_lexer::config_changed()
 	reloadpalette();
 }
 
-vcp ht_html_syntax_lexer::getcolor_syntax(UINT pal_index)
+vcp ht_html_syntax_lexer::getcolor_syntax(uint pal_index)
 {
 //	return getcolorv(&c_pal, pal_index);
 	return VCP(VC_LIGHT(VC_BLUE), VC_TRANSPARENT);
@@ -596,12 +601,12 @@ lexer_state ht_html_syntax_lexer::getinitstate()
 	return LEX_HTMLST_NORMAL;
 }
 
-char *ht_html_syntax_lexer::getname()
+const char *ht_html_syntax_lexer::getname()
 {
 	return "HTML";
 }
 
-lexer_token ht_html_syntax_lexer::gettoken(void *buf, UINT buflen, text_pos p, bool start_of_line, lexer_state *ret_state, UINT *ret_len)
+lexer_token ht_html_syntax_lexer::gettoken(void *buf, uint buflen, text_pos p, bool start_of_line, lexer_state *ret_state, uint *ret_len)
 {
 	lexer_token t=ht_lang_syntax_lexer::gettoken(buf, buflen, p, start_of_line, ret_state, ret_len);
 /*	if (t==LEX_CTOK_IDENTIFIER) {
@@ -647,12 +652,12 @@ int qsort_stringlist(const void *e1, const void *e2)
 	return strcmp(*(char **)e1, *(char **)e2);
 }
 	
-char **create_sorted_stringtable(char **table)
+const char **create_sorted_stringtable(const char **table)
 {
-	char **x=table;
+	const char **x=table;
 	while (*x) x++;
-	char **stab=(char **)malloc(sizeof (char*) * (x-table+1));
-	memmove(stab, table, sizeof (char*) * (x-table+1));
+	const char **stab = ht_malloc(sizeof (char*) * (x-table+1));
+	memcpy(stab, table, sizeof (char*) * (x-table+1));
 	
 	qsort(stab, x-table, sizeof(char*), qsort_stringlist);
 	return stab;
