@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "except.h"
 #include "htdebug.h"
 #include "textfile.h"
 #include "tools.h"
@@ -36,82 +37,86 @@
 /*
  *	CLASS ht_textfile
  */
+ht_textfile::ht_textfile(File *file, bool own_file)
+	: FileLayer(file, own_file)
+{
+}
 
 /*
  *	CLASS ht_layer_textfile
  */
  
-void ht_layer_textfile::init(ht_textfile *textfile, bool own_textfile)
+ht_layer_textfile::ht_layer_textfile(ht_textfile *textfile, bool own_textfile)
+	: ht_textfile(textfile, own_textfile)
 {
-	ht_textfile::init(textfile, own_textfile);
 }
 
-bool ht_layer_textfile::convert_ofs2line(FILEOFS o, UINT *line, UINT *pofs)
+bool ht_layer_textfile::convert_ofs2line(FileOfs o, uint *line, uint *pofs) const
 {
-	return ((ht_textfile*)streamfile)->convert_ofs2line(o, line, pofs);
+	return ((ht_textfile*)mFile)->convert_ofs2line(o, line, pofs);
 }
 
-bool ht_layer_textfile::convert_line2ofs(UINT line, UINT pofs, FILEOFS *o)
+bool ht_layer_textfile::convert_line2ofs(uint line, uint pofs, FileOfs *o) const
 {
-	return ((ht_textfile*)streamfile)->convert_line2ofs(line, pofs, o);
+	return ((ht_textfile*)mFile)->convert_line2ofs(line, pofs, o);
 }
 
-void ht_layer_textfile::delete_lines(UINT line, UINT count)
+void ht_layer_textfile::delete_lines(uint line, uint count)
 {
-	((ht_textfile*)streamfile)->delete_lines(line, count);
+	((ht_textfile*)mFile)->delete_lines(line, count);
 }
 
-void ht_layer_textfile::delete_chars(UINT line, UINT ofs, UINT count)
+void ht_layer_textfile::delete_chars(uint line, uint ofs, uint count)
 {
-	((ht_textfile*)streamfile)->delete_chars(line, ofs, count);
+	((ht_textfile*)mFile)->delete_chars(line, ofs, count);
 }
 
-bool ht_layer_textfile::get_char(UINT line, char *ch, UINT pos)
+bool ht_layer_textfile::get_char(uint line, char *ch, uint pos)
 {
-	return ((ht_textfile*)streamfile)->get_char(line, ch, pos);
+	return ((ht_textfile*)mFile)->get_char(line, ch, pos);
 }
 
-bool ht_layer_textfile::getline(UINT line, UINT pofs, void *buf, UINT buflen, UINT *retlen, lexer_state *state)
+bool ht_layer_textfile::getline(uint line, uint pofs, void *buf, uint buflen, uint *retlen, lexer_state *state)
 {
-	return ((ht_textfile*)streamfile)->getline(line, pofs, buf, buflen, retlen, state);
+	return ((ht_textfile*)mFile)->getline(line, pofs, buf, buflen, retlen, state);
 }
 
-UINT ht_layer_textfile::getlinelength(UINT line)
+uint ht_layer_textfile::getlinelength(uint line) const
 {
-	return ((ht_textfile*)streamfile)->getlinelength(line);
+	return ((ht_textfile*)mFile)->getlinelength(line);
 }
 
-void ht_layer_textfile::insert_lines(UINT before, UINT count, void **line_ends, int *line_end_lens)
+void ht_layer_textfile::insert_lines(uint before, uint count, void **line_ends, int *line_end_lens)
 {
-	((ht_textfile*)streamfile)->insert_lines(before, count, line_ends, line_end_lens);
+	((ht_textfile*)mFile)->insert_lines(before, count, line_ends, line_end_lens);
 }
 
-void ht_layer_textfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
+void ht_layer_textfile::insert_chars(uint line, uint ofs, void *chars, uint len)
 {
-	((ht_textfile*)streamfile)->insert_chars(line, ofs, chars, len);
+	((ht_textfile*)mFile)->insert_chars(line, ofs, chars, len);
 }
 
-bool ht_layer_textfile::has_line(UINT line)
+bool ht_layer_textfile::has_line(uint line)
 {
-	return ((ht_textfile*)streamfile)->has_line(line);
+	return ((ht_textfile*)mFile)->has_line(line);
 }
 
-UINT ht_layer_textfile::linecount()
+uint ht_layer_textfile::linecount() const
 {
-	return ((ht_textfile*)streamfile)->linecount();
+	return ((ht_textfile*)mFile)->linecount();
 }
 
-void ht_layer_textfile::set_layered_assume(ht_streamfile *s, bool changes_applied)
+void ht_layer_textfile::set_layered_assume(File *s, bool ownNewLayered, bool changes_applied)
 {
-/*	ht_streamfile *q = streamfile->get_layered();
-	if (q)*/ ((ht_textfile*)streamfile)->set_layered_assume(s, changes_applied);
+/*	File *q = streamfile->get_layered();
+	if (q)*/ ((ht_textfile*)mFile)->set_layered_assume(s, ownNewLayered, changes_applied);
 /*     else
 		streamfile = s;*/
 }
 
 void ht_layer_textfile::set_lexer(ht_syntax_lexer *lexer)
 {
-	((ht_textfile*)streamfile)->set_lexer(lexer);
+	((ht_textfile*)mFile)->set_lexer(lexer);
 }
 
 /*
@@ -127,16 +132,12 @@ ht_ltextfile_line::~ht_ltextfile_line()
  *	CLASS ht_ltextfile
  */
  
-void	ht_ltextfile::init(ht_streamfile *streamfile, bool own_streamfile, ht_syntax_lexer *l)
+ht_ltextfile::ht_ltextfile(File *streamfile, bool own_streamfile, ht_syntax_lexer *l)
+	: ht_textfile(streamfile, own_streamfile)
 {
-	ht_textfile::init(streamfile, own_streamfile);
-	int e = get_error();
-	if (get_error()) return;
 	lexer = l;
-	lines = new ht_clist();
-	lines->init();
-	orig_lines = new ht_clist();
-	orig_lines->init();
+	lines = new Array(true);
+	orig_lines = new Array(true);
 	reread();
 	first_parse_dirty_line = linecount();
 	first_nofs_dirty_line = linecount();
@@ -144,24 +145,18 @@ void	ht_ltextfile::init(ht_streamfile *streamfile, bool own_streamfile, ht_synta
 	ofs = 0;
 }
 
-void ht_ltextfile::done()
+ht_ltextfile::~ht_ltextfile()
 {
-	lines->destroy();
 	delete lines;
-	orig_lines->destroy();
 	delete orig_lines;
-	ht_textfile::done();
 }
 
 void ht_ltextfile::cache_invd()
 {
-	lines->empty();
-	UINT c=orig_lines->count();
-	for (UINT i=0; i<c; i++) {
-		ht_ltextfile_line *l = new ht_ltextfile_line();
-		*l = *((ht_ltextfile_line*)orig_lines->get(i));
-		lines->append(l);
-	}
+	lines->delAll();
+	foreach(ht_ltextfile_line, l, *orig_lines, {
+		lines->insert(new ht_ltextfile_line(*l));
+	});
 	dirty_parse(0);
 /*	reread();*/
 	dirty = false;
@@ -171,12 +166,12 @@ void ht_ltextfile::cache_flush()
 {
 }
 
-bool ht_ltextfile::convert_line2ofs(UINT line, UINT pofs, FILEOFS *o)
+bool ht_ltextfile::convert_line2ofs(uint line, uint pofs, FileOfs *o) const
 {
 	ht_ltextfile_line *x;
 	x=fetch_line_nofs_ok(line);
 	if (x) {
-		UINT m=getlinelength_i(x);
+		uint m=getlinelength_i(x);
 		if (pofs<m) {
 			*o=x->nofs+pofs;
 		} else {
@@ -187,11 +182,11 @@ bool ht_ltextfile::convert_line2ofs(UINT line, UINT pofs, FILEOFS *o)
 	return false;
 }
 
-bool ht_ltextfile::convert_ofs2line(FILEOFS o, UINT *line, UINT *pofs)
+bool ht_ltextfile::convert_ofs2line(FileOfs o, uint *line, uint *pofs) const
 {
-	UINT l=0, r=linecount();
+	uint l=0, r=linecount();
 	if (!r) return 0; else r--;
-	UINT m=0;
+	uint m=0;
 	ht_ltextfile_line *x, *y;
 	while (l<=r) {
 		m=(l+r) / 2;
@@ -215,40 +210,43 @@ bool ht_ltextfile::convert_ofs2line(FILEOFS o, UINT *line, UINT *pofs)
 	return true;
 }
 
-void	ht_ltextfile::copy_to(ht_stream *stream)
+FileOfs ht_ltextfile::copyAllTo(Stream *stream)
 {
-/*	UINT c=linecount();
+/*	uint c=linecount();
 	char buf[TEXTFILE_MAX_LINELEN+1];
-	for (UINT i=0; i<c; i++) {
+	for (uint i=0; i<c; i++) {
 		ht_ltextfile_line *e=fetch_line(i);
 		if (e) {
-			UINT s;
+			uint s;
 			getline(i, 0, buf, TEXTFILE_MAX_LINELEN+1, &s, NULL);
 			stream->write(buf, s);
 			stream->write(e->lineend, e->lineendlen);
 		}
 	}*/
 #define STREAM_COPYBUF_SIZE (64*1024)
-	const UINT bufsize=STREAM_COPYBUF_SIZE;
-	byte *buf=(byte*)malloc(bufsize);
-	UINT r;
+	const uint bufsize=STREAM_COPYBUF_SIZE;
+	byte *buf = ht_malloc(bufsize);
+	uint r;
+	FileOfs res = 0;
 	do {
-		r=read(buf, bufsize);
-		stream->write(buf, r);
+		r = read(buf, bufsize);
+		stream->writex(buf, r);
+		res += r;
 	} while (r);
 	free(buf);
+	return res;
 }
 
-void ht_ltextfile::delete_chars(UINT line, UINT ofs, UINT count)
+void ht_ltextfile::delete_chars(uint line, uint ofs, uint count)
 {
 	ht_ltextfile_line *e=fetch_line_into_memory(line);
 	if (e) {
-		char *ostr=e->in_memory.data;
-		UINT olen=e->in_memory.len;
+		char *ostr = e->in_memory.data;
+		uint olen = e->in_memory.len;
 		
-		if (ofs<olen) {
+		if (ofs < olen) {
 			if (ofs+count>olen) count=olen-ofs;
-			char *nstr=(char*)malloc(olen-count);
+			char *nstr = ht_malloc(olen-count);
 			memcpy(nstr, ostr, ofs);
 			memcpy(nstr+ofs, ostr+ofs+count, olen-ofs-count);
 			free(ostr);
@@ -261,52 +259,54 @@ void ht_ltextfile::delete_chars(UINT line, UINT ofs, UINT count)
 	}
 }
 
-void ht_ltextfile::delete_lines(UINT line, UINT count)
+void ht_ltextfile::delete_lines(uint line, uint count)
 {
-	lines->del_multiple(line, count);
-	dirty_parse(line);
-	dirty_nofs(line);
-	dirty=true;
+	if (count) {
+		lines->delRange(line, line+count-1);
+		dirty_parse(line);
+		dirty_nofs(line);
+		dirty=true;
+	}
 }
 
-void ht_ltextfile::dirty_parse(UINT line)
+void ht_ltextfile::dirty_parse(uint line)
 {
 	if (line<first_parse_dirty_line) first_parse_dirty_line=line;
 }
 
-void ht_ltextfile::dirty_nofs(UINT line)
+void ht_ltextfile::dirty_nofs(uint line)
 {
 	if (line<first_nofs_dirty_line) first_nofs_dirty_line=line;
 }
 
-int	ht_ltextfile::extend(UINT newsize)
+void ht_ltextfile::extend(FileOfs newsize)
 {
 	/* FIXME: nyi */
-	return ENOSYS;
+	throw IOException(ENOSYS);
 }
 
-ht_ltextfile_line *ht_ltextfile::fetch_line(UINT line)
+ht_ltextfile_line *ht_ltextfile::fetch_line(uint line) const
 {
-	return (ht_ltextfile_line*)lines->get(line);
+	return (ht_ltextfile_line*)(*lines)[line];
 }
 
-ht_ltextfile_line *ht_ltextfile::fetch_line_nofs_ok(UINT line)
+ht_ltextfile_line *ht_ltextfile::fetch_line_nofs_ok(uint line) const
 {
 	if (is_dirty_nofs(line)) update_nofs(line);
 	return fetch_line(line);
 }
 			
-UINT ht_ltextfile::find_linelen_forwd(byte *buf, UINT maxbuflen, FILEOFS ofs, int *le_len)
+uint ht_ltextfile::find_linelen_forwd(byte *buf, uint maxbuflen, FileOfs ofs, int *le_len)
 {
-	UINT readlen=(maxbuflen>TEXTFILE_READSIZE) ? TEXTFILE_READSIZE : maxbuflen;
+	uint readlen=(maxbuflen>TEXTFILE_READSIZE) ? TEXTFILE_READSIZE : maxbuflen;
 	byte *bufp;
-	UINT s;
-	UINT len = 0;
+	uint s;
+	uint len = 0;
 	
 	if (le_len) *le_len = 0;
 	do {
-		streamfile->seek(ofs);
-		s = streamfile->read(buf, readlen);
+		mFile->seek(ofs);
+		s = mFile->read(buf, readlen);
 		int l;
 		bufp = match_lineend_forwd(buf, s, &l);
 		if (bufp) {
@@ -332,14 +332,14 @@ UINT ht_ltextfile::find_linelen_forwd(byte *buf, UINT maxbuflen, FILEOFS ofs, in
 	return len;
 }
 
-ht_ltextfile_line *ht_ltextfile::fetch_line_into_memory(UINT line)
+ht_ltextfile_line *ht_ltextfile::fetch_line_into_memory(uint line)
 {
 	ht_ltextfile_line *e=fetch_line(line);
 	if (e) {
 		if (!e->is_in_memory) {
-			char *data=(char*)malloc(e->on_disk.len);
-			streamfile->seek(e->on_disk.ofs);
-			UINT x=streamfile->read(data, e->on_disk.len);
+			char *data = ht_malloc(e->on_disk.len);
+			mFile->seek(e->on_disk.ofs);
+			uint x=mFile->read(data, e->on_disk.len);
 
 			e->is_in_memory=true;
 			e->in_memory.data=data;
@@ -349,15 +349,15 @@ ht_ltextfile_line *ht_ltextfile::fetch_line_into_memory(UINT line)
 	return e;
 }
 
-UINT	ht_ltextfile::get_size()
+FileOfs	ht_ltextfile::getSize() const
 {
 	int line = linecount()-1;
-	FILEOFS o = 0;
+	FileOfs o = 0;
 	convert_line2ofs(line, getlinelength(line), &o);
 	return o;
 }
 
-bool ht_ltextfile::get_char(UINT line, char *ch, UINT pos)
+bool ht_ltextfile::get_char(uint line, char *ch, uint pos)
 {
 	ht_ltextfile_line *e=fetch_line(line);
 	if (e) {
@@ -368,8 +368,8 @@ bool ht_ltextfile::get_char(UINT line, char *ch, UINT pos)
 			}
 		} else {
 			if (pos<e->on_disk.len) {
-				streamfile->seek(e->on_disk.ofs);
-				streamfile->read(ch, 1);
+				mFile->seek(e->on_disk.ofs);
+				mFile->readx(ch, 1);
 				return true;
 			}
 		}
@@ -378,7 +378,7 @@ bool ht_ltextfile::get_char(UINT line, char *ch, UINT pos)
 }
 
 
-bool ht_ltextfile::getline(UINT line, UINT pofs, void *buf, UINT buflen, UINT *retlen, lexer_state *state)
+bool ht_ltextfile::getline(uint line, uint pofs, void *buf, uint buflen, uint *retlen, lexer_state *state)
 {
 /* <debug> */
 /*	bool debug=false;
@@ -402,19 +402,19 @@ bool ht_ltextfile::getline(UINT line, UINT pofs, void *buf, UINT buflen, UINT *r
 
 /* </debug> */
 
-		UINT ret;
+		uint ret;
 		if (e->is_in_memory) {
-			UINT l=e->in_memory.len;
+			uint l=e->in_memory.len;
 			if (l>buflen-1) l=buflen-1;
 			if (pofs>l) l=0; else l-=pofs;
-			memmove(buf, e->in_memory.data+pofs, l);
+			memcpy(buf, e->in_memory.data+pofs, l);
 			ret=l;
 		} else {
-			UINT l=e->on_disk.len;
+			uint l=e->on_disk.len;
 			if (l>buflen-1) l=buflen-1;
 			if (pofs>l) l=0; else l-=pofs;
-			streamfile->seek(e->on_disk.ofs+pofs);
-			streamfile->read(buf, l);
+			mFile->seek(e->on_disk.ofs+pofs);
+			mFile->read(buf, l);
 			ret=l;
 		}
 		if (state) *state=e->instate;
@@ -424,13 +424,13 @@ bool ht_ltextfile::getline(UINT line, UINT pofs, void *buf, UINT buflen, UINT *r
 	return false;
 }
 
-UINT ht_ltextfile::getlinelength(UINT line)
+uint ht_ltextfile::getlinelength(uint line) const
 {
 	ht_ltextfile_line *e=fetch_line(line);
 	return getlinelength_i(e);
 }
 
-UINT ht_ltextfile::getlinelength_i(ht_ltextfile_line *e)
+uint ht_ltextfile::getlinelength_i(ht_ltextfile_line *e) const
 {
 	if (e) {
 		if (e->is_in_memory) {
@@ -442,7 +442,7 @@ UINT ht_ltextfile::getlinelength_i(ht_ltextfile_line *e)
 	return 0;
 }
 
-void ht_ltextfile::insert_lines(UINT before, UINT count, void **line_ends, int *line_end_lens)
+void ht_ltextfile::insert_lines(uint before, uint count, void **line_ends, int *line_end_lens)
 {
 	ht_ltextfile_line *e = fetch_line(before);
 	int instate = e->instate;
@@ -452,40 +452,40 @@ void ht_ltextfile::insert_lines(UINT before, UINT count, void **line_ends, int *
 		e->instate = instate;
 		e->on_disk.ofs = 0xffffffff;
 		e->on_disk.len = 0;
-		e->in_memory.data = (char*)malloc(1);
+		e->in_memory.data = ht_malloc(1);
 		e->in_memory.len = 0;
 		e->nofs = 0;
 		if (line_ends && line_end_lens) {
 			e->lineendlen = *line_end_lens++;
-			memmove(e->lineend, *line_ends++, e->lineendlen);
+			memcpy(e->lineend, *line_ends++, e->lineendlen);
 		} else {
 			e->lineendlen = 1;
 			e->lineend[0] = '\n';
 		}
-		lines->insert_before(e, before);
+		lines->insertBefore(before, e);
 	}
 	dirty_parse(before);
 	dirty_nofs(before);
 	dirty = true;
 }
 
-void ht_ltextfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
+void ht_ltextfile::insert_chars(uint line, uint ofs, void *chars, uint len)
 {
 	ht_ltextfile_line *e=fetch_line(line);
 	if (e) {
-		UINT olen=e->is_in_memory ? e->in_memory.len : e->on_disk.len;
+		uint olen=e->is_in_memory ? e->in_memory.len : e->on_disk.len;
 		if (ofs<=olen) {
 			e=fetch_line_into_memory(line);
 			char *ostr=e->in_memory.data;
 
 			char *nstr;
-			UINT nlen=olen;
+			uint nlen=olen;
 
-			UINT clen=len;
+			uint clen=len;
 
 			if (ofs>nlen) nlen=ofs;
 			nlen+=clen;
-			nstr=(char*)malloc(nlen);
+			nstr = ht_malloc(nlen);
 
 			memcpy(nstr, ostr, ofs);
 			memcpy(nstr+ofs, chars, clen);
@@ -501,27 +501,27 @@ void ht_ltextfile::insert_chars(UINT line, UINT ofs, void *chars, UINT len)
 	}
 }
 
-bool ht_ltextfile::has_line(UINT line)
+bool ht_ltextfile::has_line(uint line)
 {
 	return (fetch_line(line)!=NULL);
 }
 
-bool ht_ltextfile::is_dirty_nofs(UINT line)
+bool ht_ltextfile::is_dirty_nofs(uint line) const
 {
-	return (line>=first_nofs_dirty_line);
+	return (line >= first_nofs_dirty_line);
 }
 
-bool ht_ltextfile::is_dirty_parse(UINT line)
+bool ht_ltextfile::is_dirty_parse(uint line) const
 {
-	return (line>=first_parse_dirty_line);
+	return (line >= first_parse_dirty_line);
 }
 
-UINT ht_ltextfile::linecount()
+uint ht_ltextfile::linecount() const
 {
 	return lines->count();
 }
 
-byte *ht_ltextfile::match_lineend_forwd(byte *buf, UINT buflen, int *le_len)
+byte *ht_ltextfile::match_lineend_forwd(byte *buf, uint buflen, int *le_len)
 {
 	byte *result=NULL;
 	
@@ -538,19 +538,19 @@ byte *ht_ltextfile::match_lineend_forwd(byte *buf, UINT buflen, int *le_len)
 	return result;
 }
 
-lexer_state ht_ltextfile::next_instate(UINT line)
+lexer_state ht_ltextfile::next_instate(uint line)
 {
 	byte buf[TEXTFILE_MAX_LINELEN+1];
 	lexer_state state = 0;
 
-	UINT buflen;
+	uint buflen;
 	if (!getline(line, 0, buf, TEXTFILE_MAX_LINELEN, &buflen, &state)) return state;
 	buf[buflen] = 0;
 	
 	if (lexer) {
 		text_pos p;
 		char *bufp = (char*)buf;
-		UINT toklen;
+		uint toklen;
 		bool start_of_line = true;
 		p.line = line;
 		p.pofs = 0;
@@ -568,7 +568,7 @@ lexer_state ht_ltextfile::next_instate(UINT line)
 	return state;
 }
 
-FILEOFS ht_ltextfile::next_nofs(ht_ltextfile_line *l)
+FileOfs ht_ltextfile::next_nofs(ht_ltextfile_line *l) const
 {
 	if (l) {
 		if (l->is_in_memory) return l->nofs+l->in_memory.len+l->lineendlen; else
@@ -577,29 +577,28 @@ FILEOFS ht_ltextfile::next_nofs(ht_ltextfile_line *l)
 	return 0;
 }
 
-void	ht_ltextfile::pstat(pstat_t *s)
+void	ht_ltextfile::pstat(pstat_t &s) const
 {
-	streamfile->pstat(s);
-	s->size=get_size();
-	s->size_high=0;
+	mFile->pstat(s);
+	s.size = getSize();
 }
 
-UINT	ht_ltextfile::read(void *buf, UINT size)
+uint	ht_ltextfile::read(void *buf, uint size)
 {
-	FILEOFS o=tell();
-	UINT line;
-	UINT pofs;
-	UINT c=0;
+	FileOfs o=tell();
+	uint line;
+	uint pofs;
+	uint c=0;
 	byte *b=(byte*)buf;
 	
 	if (convert_ofs2line(o, &line, &pofs)) while (size) {
 		ht_ltextfile_line *l=fetch_line(line);
 		if (!l)
 			break;
-		UINT q;
+		uint q;
 		if (l->is_in_memory) {
 			q=l->in_memory.len;
-			UINT s=q;
+			uint s=q;
 			if (s>pofs) {
 				s-=pofs;
 				s=MIN(size, s);
@@ -610,20 +609,20 @@ UINT	ht_ltextfile::read(void *buf, UINT size)
 			}
 		} else {
 			q=l->on_disk.len;
-			UINT r;
-			UINT s=q;
+			uint r;
+			uint s=q;
 			if (s>pofs) {
 				s-=pofs;
 				s=MIN(size, s);
-				streamfile->seek(l->on_disk.ofs+pofs);
-				r=streamfile->read(b+c, s);
+				mFile->seek(l->on_disk.ofs+pofs);
+				r = mFile->read(b+c, s);
 				if (r!=s) break;
 				size-=s;
 				c+=s;
 				pofs+=s;
 			}
 		}
-		UINT s=l->lineendlen;
+		uint s=l->lineendlen;
 		if ((q+s>pofs) && (pofs >= q)) {
 			s-=pofs-q;
 			s=MIN(size, s);
@@ -640,12 +639,12 @@ UINT	ht_ltextfile::read(void *buf, UINT size)
 
 void ht_ltextfile::reread()
 {
-	lines->empty();
-	orig_lines->empty();
-	dirty=false;
-	FILEOFS ofs=0;
+	lines->delAll();
+	orig_lines->delAll();
+	dirty = false;
+	FileOfs ofs=0;
 	int ll, pll=-1, ln=0;
-	bool firstline=true;
+	bool firstline = true;
 	byte buf[TEXTFILE_MAX_LINELEN+1];
 
 	lexer_state state=0;
@@ -653,10 +652,10 @@ void ht_ltextfile::reread()
 
 	if (lexer) state=lexer->getinitstate();
 
-	UINT l=0;
+	uint l=0;
 	while ((l=find_linelen_forwd(buf, sizeof buf, ofs, &ll))) {
-		if (streamfile->seek(ofs) != 0) break;
-		UINT x=streamfile->read(buf, l-ll);
+		mFile->seek(ofs);
+		uint x = mFile->read(buf, l-ll);
 		buf[x]=0;
 
 		e=new ht_ltextfile_line();
@@ -666,17 +665,17 @@ void ht_ltextfile::reread()
 		e->on_disk.len=l-ll;
 		e->nofs=ofs;
 		e->lineendlen=ll;
-		assert( (UINT)ll <= sizeof e->lineend);
-		streamfile->read(e->lineend, ll);
-		lines->append(e);
+		assert( (uint)ll <= sizeof e->lineend);
+		mFile->read(e->lineend, ll);
+		lines->insert(e);
 		ce=new ht_ltextfile_line();
 		*ce = *e;
-		orig_lines->append(ce);
+		orig_lines->insert(ce);
 
 		if (lexer) {
 			text_pos p;
 			char *bufp=(char*)buf;
-			UINT toklen;
+			uint toklen;
 			bool start_of_line=true;
 			p.line=ln;
 			p.pofs=0;
@@ -702,22 +701,22 @@ void ht_ltextfile::reread()
 		e->on_disk.len=l-ll;
 		e->nofs=ofs;
 		e->lineendlen=ll;
-		lines->append(e);
+		lines->insert(e);
 		ce=new ht_ltextfile_line();
 		*ce = *e;
-		orig_lines->append(ce);
+		orig_lines->insert(ce);
 	}
 }
 
-void ht_ltextfile::split_line(UINT a, UINT pos, void *line_end, int line_end_len)
+void ht_ltextfile::split_line(uint a, uint pos, void *line_end, int line_end_len)
 {
 	if (pos == 0) {
 		insert_lines(a, 1, &line_end, &line_end_len);
 	} else {
-		UINT l=getlinelength(a);
-		if (pos>l) pos=l;
-		char *aline=(char*)malloc(pos+1);
-		UINT alinelen;
+		uint l=getlinelength(a);
+		if (pos > l) pos = l;
+		char *aline = ht_malloc(pos+1);
+		uint alinelen;
 		getline(a, 0, aline, pos+1, &alinelen, NULL);
 	
 		insert_lines(a, 1, &line_end, &line_end_len);
@@ -728,24 +727,23 @@ void ht_ltextfile::split_line(UINT a, UINT pos, void *line_end, int line_end_len
 	}
 }
 
-int	ht_ltextfile::seek(FILEOFS offset)
+void ht_ltextfile::seek(FileOfs offset)
 {
 	ofs = offset;
-	return 0;
 }
 
-void ht_ltextfile::set_layered(ht_streamfile *streamfile)
+void ht_ltextfile::setLayered(File *streamfile, bool ownNewLayered)
 {
-	ht_layer_streamfile::set_layered(streamfile);
+	FileLayer::setLayered(streamfile, ownNewLayered);
 	reread();
 }
 
-void ht_ltextfile::set_layered_assume(ht_streamfile *streamfile, bool changes_applied)
+void ht_ltextfile::set_layered_assume(File *streamfile, bool ownNewLayered, bool changes_applied)
 {
 	if (changes_applied) {
-		orig_lines->empty();
-		UINT c = lines->count();
-		for (UINT i=0; i<c; i++) {
+		orig_lines->delAll();
+		uint c = lines->count();
+		for (uint i=0; i<c; i++) {
 			ht_ltextfile_line *l = fetch_line_nofs_ok(i);
 			ht_ltextfile_line *m = new ht_ltextfile_line();
 			l->on_disk.ofs = l->nofs;
@@ -755,11 +753,11 @@ void ht_ltextfile::set_layered_assume(ht_streamfile *streamfile, bool changes_ap
 				l->is_in_memory = false;
 			}
 			*m = *l;
-			orig_lines->append(m);
+			orig_lines->insert(m);
 		}
 		cache_invd();
 	}
-	ht_layer_streamfile::set_layered(streamfile);
+	FileLayer::setLayered(streamfile, ownNewLayered);
 }
 
 void ht_ltextfile::set_lexer(ht_syntax_lexer *l)
@@ -770,21 +768,21 @@ void ht_ltextfile::set_lexer(ht_syntax_lexer *l)
 	}
 }
 
-FILEOFS ht_ltextfile::tell()
+FileOfs ht_ltextfile::tell() const
 {
 	return ofs;
 }
 
-int	ht_ltextfile::truncate(UINT newsize)
+void	ht_ltextfile::truncate(FileOfs newsize)
 {
 	/* FIXME: nyi */
-	return ENOSYS;
+	throw IOException(ENOSYS);
 }
 
-void ht_ltextfile::update_parse(UINT target)
+void ht_ltextfile::update_parse(uint target)
 {
 	ht_ltextfile_line *e;
-	UINT line = first_parse_dirty_line;
+	uint line = first_parse_dirty_line;
 
 	lexer_state instate=0;
 	if (line) {
@@ -809,12 +807,12 @@ void ht_ltextfile::update_parse(UINT target)
 	if (e) e->instate = instate;
 }
 
-void ht_ltextfile::update_nofs(UINT target)
+void ht_ltextfile::update_nofs(uint target) const
 {
 	ht_ltextfile_line *e;
-	UINT line=first_nofs_dirty_line;
+	uint line=first_nofs_dirty_line;
 
-	FILEOFS nofs;
+	FileOfs nofs;
 	if (line) {
 		e=fetch_line(line-1);
 		nofs=next_nofs(e);
@@ -832,44 +830,47 @@ void ht_ltextfile::update_nofs(UINT target)
 	}
 }
 
-int ht_ltextfile::vcntl(UINT cmd, va_list vargs)
+int ht_ltextfile::vcntl(uint cmd, va_list vargs)
 {
 	switch (cmd) {
-		case FCNTL_MODS_INVD:
-				cache_invd();
-				return 0;
-		case FCNTL_MODS_FLUSH:
-				cache_flush();
-				return 0;
-		case FCNTL_MODS_IS_DIRTY: {
-				FILEOFS o=va_arg(vargs, FILEOFS);
-				UINT s=va_arg(vargs, UINT);
-				bool *b=va_arg(vargs, bool*);
-				*b = dirty;
-				o = 0;	// gcc warns otherwise
-				s = 0;    // gcc warns otherwise
-				return 0;
-		}
-		case FCNTL_MODS_CLEAR_DIRTY: {
-			dirty = false;
-			return 0;
-		}
+	case FCNTL_MODS_INVD:
+		cache_invd();
+		return 0;
+	case FCNTL_MODS_FLUSH:
+		cache_flush();
+		return 0;
+	case FCNTL_MODS_IS_DIRTY: {
+		FileOfs o = va_arg(vargs, FileOfs);
+		FileOfs s = va_arg(vargs, FileOfs);
+		bool *b = va_arg(vargs, bool*);
+		*b = dirty;
+		o = 0;	// gcc warns otherwise
+		s = 0;    // gcc warns otherwise
+		return 0;
+	}
+	case FCNTL_MODS_CLEAR_DIRTY_RANGE: {
+		FileOfs o = va_arg(vargs, FileOfs);
+		FileOfs s = va_arg(vargs, FileOfs);
+		dirty = false;
+		o = s = 0;
+		return 0;
+	}
 	}
 	return ht_textfile::vcntl(cmd, vargs);
 }
 
-UINT	ht_ltextfile::write(const void *buf, UINT size)
+uint	ht_ltextfile::write(const void *buf, uint size)
 {
-	FILEOFS o = tell();
-	UINT line;
-	UINT pofs;
+	FileOfs o = tell();
+	uint line;
+	uint pofs;
 	byte *b = (byte*)buf;
-	UINT r = 0;
+	uint r = 0;
 	if (convert_ofs2line(o, &line, &pofs)) {
 		r = size;
 		while (size) {
 			int lelen;
-			UINT s;
+			uint s;
 			byte *c = match_lineend_forwd(b, size, &lelen);
 			if (c) {
 				s = c-b;
